@@ -10,12 +10,20 @@ from dwh_oppfolging.transforms.functions import (
     json_to_string,
     string_to_sha256_hash,
     string_to_naive_norwegian_datetime,
-    naive_norwegian_datetime_to_naive_utc0_datetime,
 )
 
 API_VERSION = 2
 API_NAME = "BRREG"
 
+
+class UnitNotFoundError(Exception):
+    """Unit not found"""
+
+
+class UnitHasNoUpdatesError(Exception):
+    """Unit has no updates"""
+
+
 class BrregUnitAPI:
     """Base class for BRREG endpoints of Enheter and Underenheter"""
     def __init__(self, unit_type: Literal['Enhet', 'Underenhet'], download_date: datetime) -> None:
@@ -85,13 +93,33 @@ class BrregUnitAPI:
         return date_str
 
 
-    def get_unit(self, orgnr: str):
+    def make_fake_unit(self, orgnr: str):
         """
-        Makes a get request to fetch the unit with given orgnr.
-        
+        Makes a fake unit json document
+
         Params:
             - orgnr: 9-digit organization string
 
+        Returns:
+            - json document
+        """
+        return {"organisasjonsnummer": orgnr}
+
+
+    def get_unit(self, orgnr: str, fake_if_not_found: bool = False):
+        """
+        Makes a get request to fetch the unit with given orgnr.
+
+        If fake_if_not_found is True and the unit is not found,
+        a faked unit document is returned.
+
+        Params:
+            - orgnr: 9-digit organization string
+            - fake_if_not_found: bool
+
+        Raises:
+            - UnitNotFoundError
+
         Returns:
             - json document
         """
@@ -101,11 +129,23 @@ class BrregUnitAPI:
         except requests.HTTPError:
             # Note: According to the BRREG API documentation, status code 410 is used for deleted orgnr, but we actually get 404.
             logging.warning(f"orgnr {orgnr} may have been deleted")
-        document = response.json()
+        try:
+            document = response.json()
+        except requests.JSONDecodeError as exc:
+            if fake_if_not_found:
+                logging.warning(f"orgnr {orgnr} is entirely gone, faking json document")
+                document = self.make_fake_unit(orgnr)
+            else:
+                raise UnitNotFoundError(f"{orgnr}") from exc
         return document
 
 
-    def make_fake_unit_update(self, orgnr: str, last_modified_date: datetime):
+    def make_fake_unit_update(
+        self,
+        orgnr: str,
+        change: str = 'UKJENT',
+        last_modified_date: datetime = datetime(1899, 12, 31, 23),
+    ):
         """
         Creates a fake unit update json document.
         Useful for units which have no update history.
@@ -113,23 +153,31 @@ class BrregUnitAPI:
         Params:
             - orgnr: 9-digit organization string
             - last_modified_date: naive UTC0 datetime
-        
+                default such that row will be generated with datetime 01.01.1900
+
         Returns:
             - json document
         """
         document = {
             "organisasjonsnummer": orgnr,
-            "endringstype": "UKJENT",
+            "endringstype": change,
             "dato": self.naive_utc0_datetime_to_brreg_date_str(last_modified_date)
         }
         return document
 
 
-    def get_unit_update_history(self, orgnr: str, latest_only: bool = False):
+    def get_unit_update_history(
+        self,
+        orgnr: str,
+        latest_only: bool = False,
+        fake_if_not_found: bool = False
+    ):
         """
         Makes a get request to fetch all the updates on the unit with the given orgnr.
 
-        If latest_only is True, then only the latest update (if any) is returned.
+        If latest_only is True, then only the latest update is kept in the return.
+
+        If fake_if_not_found is True and no updates are found, a faked update is made.
 
         Params:
             - orgnr: 9-digit organization string
@@ -137,10 +185,10 @@ class BrregUnitAPI:
 
         Raises:
             - HTTPError
+            - UnitHasNoUpdatesError
 
         Returns:
-            - (possibly empty) list of json documents
-            - OR single json document (possibly None) if latest_only is True
+            - list of json documents
         """
         params = {"organisasjonsnummer": orgnr, "oppdateringsid": 1}
         all_updates = []
@@ -158,18 +206,26 @@ class BrregUnitAPI:
             params["oppdateringsid"] = updates[-1]["oppdateringsid"] + 1
             all_updates.extend(updates)
 
+        if len(all_updates) == 0:
+            if fake_if_not_found:
+                logging.warning(f"orgnr {orgnr} has no updates, faking update")
+                all_updates.append(self.make_fake_unit_update(orgnr))
+            else:
+                raise UnitHasNoUpdatesError(f"{orgnr}")
         if latest_only:
-            if len(all_updates) > 0:
-                return all_updates[-1]
-            return None
+            return all_updates[-1:]
         return all_updates
 
 
-    def get_all_updates_since(self, last_modified_date: datetime, latest_only: bool = False):
+    def get_all_updates_since(
+        self,
+        last_modified_date: datetime,
+        latest_only: bool = False
+    ):
         """
         Makes a get request to fetch all the updates on any units since the provided datetime.
 
-        If latest_only is True, then only the latest update (if any) is returned for each orgnr.
+        If latest_only is True, then only the latest update is kept for each unit.
 
         NOTE: The returned dictionary may be quite large.
 
@@ -214,7 +270,7 @@ class BrregUnitAPI:
     def make_row(self, update: dict, fact: dict):
         """
         Combines the update and fact json documents (dicts) returned for each orgnr
-        from the get_unit_update_history/get_all_unit_updates and get_unit
+        from the get_unit_update_history/get_all_unit_updates_since and get_unit
         methods, respectively.
 
         NOTE: This method may remove keys in the input dicts.
@@ -224,15 +280,13 @@ class BrregUnitAPI:
             - fact: dict
             - download_date: naive norwegian datetime
 
-        Raises:
-            - HTTPError
-        
         Returns:
             - dict
         """
         fact.pop("_links", None)
-        fact.get("organisasjonsform", {}).pop("_links", None)
         fact.pop("links", None)
+        fact.get("organisasjonsform", {}).pop("_links", None)
+        fact.get("organisasjonsform", {}).pop("links", None)
         record = {}
         record["organisasjonsnummer"] = update["organisasjonsnummer"]
         record["endringstype"] = update["endringstype"]
@@ -244,56 +298,48 @@ class BrregUnitAPI:
         record["kildesystem"] = API_NAME
         return record
 
-
-    def get_specific_units_as_rows(
-            self,
-            orgnr_list: list[str],
-            force_fake_update_if_none_found: bool = False,
-            fake_last_modified_date: datetime = datetime(1900, 1, 1),
-        ):
+    def get_unit_as_row(
+        self,
+        orgnr: str,
+        fake_update_if_not_found: bool = False,
+        fake_unit_if_not_found: bool = False,
+    ):
         """
-        Gets unit data as rows using get_update_history and get_unit methods.
-        Only the latest update for each unit is used to make the rows.
+        Gets unit data as row using get_unit_update_history and get_unit methods.
+        Only the latest update for each unit is used to make the row.
         This method is useful for finding organizations that are referenced by others,
         but haven't been updated in a long while.
 
-        NOTE: If no update is found for the given orgnr, and
-        force_fake_update_if_none_found is not True,
-        then no row is returned for that orgnr.
-
-        If the unit details/facts are not found, an empty dict {} is used.
-
         Params:
-            - orgnr_list: list of 9-digit organization strings
-            - force_fake_update_if_none_found: bool
-            - fake_last_modified_date: naive UTC0 datetime
+            - orgnr: 9-digit organization strings
+            - fake_update_if_not_found: bool
+            - fake_unit_if_not_found: bool
 
         Returns:
-            - list of rows as dicts
+            - dict
         """
-        rows = []
-        for orgnr in orgnr_list:
-            # force type, it doest return list[update] if 2nd arg True
-            update: dict | None = self.get_unit_update_history(orgnr, True) # type: ignore
-            if force_fake_update_if_none_found and update is None:
-                update = self.make_fake_unit_update(orgnr, fake_last_modified_date)
-            if update is not None:
-                fact = self.get_unit(orgnr)
-                rows.append(self.make_row(update, fact))
-
-        return rows
+        update = self.get_unit_update_history(orgnr, True, fake_update_if_not_found)[0]
+        fact = self.get_unit(orgnr, fake_unit_if_not_found)
+        row = self.make_row(update, fact)
+        return row
 
 
-    def get_all_units_as_rows_updated_since(self, last_modified_date: datetime):
+    def get_all_units_as_rows_updated_since(
+        self,
+        last_modified_date: datetime,
+        fake_unit_if_not_found: bool = False
+    ):
         """
         Gets unit data as rows using get_all_updates_since and get_unit methods.
         Only the latest update for each unit is used to make the rows.
-        
-        NOTE: If the unit details/facts are not found, an empty dict is used.
+
+        If fake_unit_if_not_found is True then a faked unit is used in the row
+        if the unit is gone despite an update having been found.
+        This situation seems unlikely.
 
         Params:
             - last_modified_date: naive datetime in UTC0 (+Z) (+00:00)
-            - download_date: naive norwegian datetime
+            - fake_unit_if_not_found: bool
 
         Returns:
             - list of rows as dicts
@@ -302,7 +348,7 @@ class BrregUnitAPI:
         orgnr_update_map = self.get_all_updates_since(last_modified_date, True)
 
         rows = [
-            self.make_row(updates[0], self.get_unit(orgnr))
+            self.make_row(updates[0], self.get_unit(orgnr, fake_unit_if_not_found))
             for orgnr, updates in orgnr_update_map.items()
         ]
 
@@ -318,6 +364,9 @@ class BrregUnitAPI:
         Must be run some time after 5 brreg local time on a given day.
         Useful for init loading table.
 
+        NOTE: The file API does not give any information about updates,
+        so all updates are faked in making rows.
+
         Params:
             - download_date: naive norwegian datetime
 
@@ -327,9 +376,6 @@ class BrregUnitAPI:
         logging.info("requesting filestream from api")
         url = self._unit_file_url
         headers = self._unit_file_headers
-        five_o_clock = datetime.today().replace(hour=5, minute=0, second=0, microsecond=0)
-        last_modified_date = naive_norwegian_datetime_to_naive_utc0_datetime(five_o_clock)
-        last_modified_date_str = self.naive_utc0_datetime_to_brreg_date_str(last_modified_date)
         with requests.get(url, headers=headers, stream=True, timeout=100) as response:
             response.raise_for_status()
             logging.info("decompressing filestream")
@@ -339,11 +385,7 @@ class BrregUnitAPI:
                 for record in ijson.items(file, "item"):  # type: ignore
                     records.append(
                         self.make_row(
-                            {
-                                "organisasjonsnummer": record["organisasjonsnummer"],
-                                "endringstype": "FLATFIL",
-                                "dato": last_modified_date_str,
-                            },
+                            self.make_fake_unit_update(record["organisasjonsnummer"],"FLATFIL"),
                             record,
                         )
                     )
