@@ -161,11 +161,15 @@ def update_table_from_sql(
     rows_affected = cur.rowcount
     num_rows_new: int = get_table_row_count(cur, schema, table)
     rows_inserted = num_rows_new - num_rows_old
+    rows_deleted = 0
+    if rows_inserted < 0:
+        rows_inserted, rows_deleted = rows_deleted, -rows_inserted
     rows_updated = rows_affected - rows_inserted
     logging.info(f"inserted {rows_inserted} new rows")
     logging.info(f"updated {rows_updated} existing rows")
+    logging.info(f"deleted {rows_deleted} rows")
     if enable_etl_logging:
-        log_etl(cur, schema, table, today, rows_inserted, rows_updated)
+        log_etl(cur, schema, table, today, rows_inserted, rows_updated, rows_deleted)
     return rows_inserted, rows_updated
 
 
